﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace HumanPC
{
    class MouseControl : IOControl
    {
        private const int INPUT_MOUSE = 0;                  // マウスイベント
        private const int INPUT_KEYBOARD = 1;               // キーボードイベント
        private const int INPUT_HARDWARE = 2;               // ハードウェアイベント

        private const int MOUSEEVENTF_MOVE = 0x1;           // マウスを移動する
        private const int MOUSEEVENTF_ABSOLUTE = 0x8000;    // 絶対座標指定

        enum MouseEvent
        {
            LEFTDOWN = 0x2,       // 左　ボタンを押す
            LEFTUP = 0x4,         // 左　ボタンを離す
            RIGHTDOWN = 0x8,      // 右　ボタンを押す
            RIGHTUP = 0x10,       // 右　ボタンを離す
            MIDDLEDOWN = 0x20,    // 中央ボタンを押す
            MIDDLEUP = 0x40,      // 中央ボタンを離す
            WHEEL = 0x800,        // ホイールを回転する
        }
        int width, height;
        public void init(int width, int height)
        {
            this.width = width;
            this.height = height;
        }

        public void move(int x, int y)
        {
            INPUT[] inputs = new INPUT[1];

            inputs[0].type = INPUT_MOUSE;
            inputs[0].mi.dwFlags = MOUSEEVENTF_MOVE | MOUSEEVENTF_ABSOLUTE;
            inputs[0].mi.dx = x * (65535 / width);
            inputs[0].mi.dy = y * (65535 / height);
            inputs[0].mi.mouseData = 0;
            inputs[0].mi.dwExtraInfo = 0;
            inputs[0].mi.time = 0;

            SendInput(1, ref inputs[0], Marshal.SizeOf(inputs[0]));
        }

        //マウスボタン
        public enum MouseButton
        {
            LEFT,
            RIGHT,
            CENTER,
        }

        private void click(MouseButton mouseButton)
        {
            int downFlag, upFlag;
            downFlag = upFlag = 0;

            switch (mouseButton)
            {
                case MouseButton.LEFT:
                    downFlag = (int)MouseEvent.LEFTDOWN;
                    upFlag = (int)MouseEvent.LEFTUP;
                    break;
                case MouseButton.RIGHT:
                    downFlag = (int)MouseEvent.RIGHTDOWN;
                    upFlag = (int)MouseEvent.RIGHTUP;
                    break;
                case MouseButton.CENTER:
                    downFlag = (int)MouseEvent.MIDDLEDOWN;
                    upFlag = (int)MouseEvent.MIDDLEUP;
                    break;
            }

            const int num = 2;
            INPUT[] inp = new INPUT[num];

            inp[0].type = INPUT_MOUSE;
            inp[0].mi.dwFlags = downFlag;
            inp[0].mi.dx = 0;
            inp[0].mi.dy = 0;
            inp[0].mi.mouseData = 0;
            inp[0].mi.dwExtraInfo = 0;
            inp[0].mi.time = 0;

            inp[1].type = INPUT_MOUSE;
            inp[1].mi.dwFlags = upFlag;
            inp[1].mi.dx = 0;
            inp[1].mi.dy = 0;
            inp[1].mi.mouseData = 0;
            inp[1].mi.dwExtraInfo = 0;
            inp[1].mi.time = 0;

            SendInput(num, ref inp[0], Marshal.SizeOf(inp[0]));
        }

        public void click2(MouseButton mouseButton, uint time)
        {
            int downFlag, upFlag;
            downFlag = upFlag = 0;

            switch (mouseButton)
            {
                case MouseButton.LEFT:
                    downFlag = (int)MouseEvent.LEFTDOWN;
                    upFlag = (int)MouseEvent.LEFTUP;
                    break;
                case MouseButton.RIGHT:
                    downFlag = (int)MouseEvent.RIGHTDOWN;
                    upFlag = (int)MouseEvent.RIGHTUP;
                    break;
                case MouseButton.CENTER:
                    downFlag = (int)MouseEvent.MIDDLEDOWN;
                    upFlag = (int)MouseEvent.MIDDLEUP;
                    break;
            }

            const int num = 2;
            INPUT[] inp = new INPUT[num];

            inp[0].type = INPUT_MOUSE;
            inp[0].mi.dwFlags = downFlag;
            inp[0].mi.dx = 0;
            inp[0].mi.dy = 0;
            inp[0].mi.mouseData = 0;
            inp[0].mi.dwExtraInfo = 0;
            inp[0].mi.time = time;

            inp[1].type = INPUT_MOUSE;
            inp[1].mi.dwFlags = upFlag;
            inp[1].mi.dx = 0;
            inp[1].mi.dy = 0;
            inp[1].mi.mouseData = 0;
            inp[1].mi.dwExtraInfo = 0;
            inp[1].mi.time = time;

            SendInput(num, ref inp[0], Marshal.SizeOf(inp[0]));
        }

        // マウスの右ボタンをクリックする
        public void leftClick()
        {
            click(MouseButton.LEFT);
        }

        // マウスの右ボタンをクリックする
        public void rightClick()
        {
            click(MouseButton.RIGHT);
        }

        //クリックのエベントを自分で処理するように
        private void click_state(MouseEvent mouseEvent)
        {
            const int num = 1;
            INPUT[] inp = new INPUT[1];

            inp[0].type = INPUT_MOUSE;
            inp[0].mi.dwFlags = (int)mouseEvent;
            inp[0].mi.dx = 0;
            inp[0].mi.dy = 0;
            inp[0].mi.mouseData = 0;
            inp[0].mi.dwExtraInfo = 0;
            inp[0].mi.time = 0;

            SendInput(num, ref inp[0], Marshal.SizeOf(inp[0]));
        }
    }
}
